#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright © 2019 Endless Mobile, Inc.
#
# This Source Code Form is subject to the terms of the Mozilla Public License,
# v. 2.0. If a copy of the MPL was not distributed with this file, You can
# obtain one at https://mozilla.org/MPL/2.0/.
#
# Alternatively, the contents of this file may be used under the terms of the
# GNU Lesser General Public License Version 2.1 or later (the "LGPL"), in
# which case the provisions of the LGPL are applicable instead of those above.
# If you wish to allow use of your version of this file only under the terms
# of the LGPL, and not to allow others to use your version of this file under
# the terms of the MPL, indicate your decision by deleting the provisions
# above and replace them with the notice and other provisions required by the
# LGPL. If you do not delete the provisions above, a recipient may use your
# version of this file under the terms of either the MPL or the LGPL.

"""Integration tests for the eos-payg-bios-pwgen utility."""

import csv
import os
import shutil
import subprocess
import tempfile
import unittest

import taptestrunner


class TestEosPaygBiosPwgen(unittest.TestCase):
    """
    Integration test for running eos-payg-bios-pwgen.

    This can be run when installed or uninstalled. When uninstalled, it
    requires G_TEST_BUILDDIR and G_TEST_SRCDIR to be set.

    The idea with this test harness is to test the eos-payg-bios-pwgen
    utility, its handling of command line arguments, its exit statuses,
    and basic sanity of its generated output.
    """

    TEST_SALT_FILE = 'test-salt.txt'

    def setUp(self):
        self.timeout_seconds = 10  # seconds per test
        self.tmpdir = tempfile.mkdtemp()
        os.chdir(self.tmpdir)
        print('tmpdir:', self.tmpdir)
	# look for the program in the source dir since it's not built
        if 'G_TEST_SRCDIR' in os.environ:
            self.__eos_payg_bios_pwgen = \
                os.path.join(os.environ['G_TEST_SRCDIR'], '..', 'eos-payg-bios-pwgen')
            self.__datadir = os.environ['G_TEST_SRCDIR']
        else:
            self.__eos_payg_bios_pwgen = os.path.join('/', 'usr', 'bin',
                    'eos-payg-bios-pwgen')
            self.__datadir = os.path.join('/', 'usr', 'share',
                    'installed-tests', 'eos-payg-bios-pwgen')
        print('eos-payg-bios-pwgen:', self.__eos_payg_bios_pwgen)
        print('data dir:', self.__datadir)

    def tearDown(self):
        shutil.rmtree(self.tmpdir)

    def runPwgen(self, *args):
        argv = [self.__eos_payg_bios_pwgen]
        argv.extend(args)
        print('Running:', argv)

        env = os.environ.copy()
        env['LC_ALL'] = 'C.UTF-8'

        info = subprocess.run(argv, timeout=self.timeout_seconds,
                              stdout=subprocess.PIPE,
                              stderr=subprocess.STDOUT,
                              env=env)
        print('Output:', info.stdout.decode('utf-8'))
        return info

    def test_help(self):
        """Test getting help."""
        info = self.runPwgen('-h')
        info.check_returncode()
        out = info.stdout.decode('utf-8').strip()
        self.assertIn('generate BIOS password for a given serial number\n', out)

    def test_help_no_args(self):
        """Test getting help if no arguments are provided."""
        info = self.runPwgen()
        self.assertIsNot(info.returncode, 0)
        out = info.stdout.decode('utf-8').strip()
        self.assertIn('usage:', out)
        self.assertIn('following arguments are required: SALT_FILE, SERIAL_NUMBER', out)

    def test_insufficient_args(self):
        """Test error handling when providing too few command-line arguments."""
        info = self.runPwgen('some-salt-file.txt')
        out = info.stdout.decode('utf-8').strip()
        self.assertIn('the following arguments are required: SERIAL_NUMBER', out)
        self.assertEqual(info.returncode, 2)

    def test_missing_salt_file(self):
        """Test error handling when providing a salt filename for a missing file."""
        salt_filename = 'non-existent-file.txt'
        info = self.runPwgen(salt_filename, "my-serial-number")
        out = info.stdout.decode('utf-8').strip()
        self.assertIn("No such file or directory: '{}'".format(salt_filename), out)
        self.assertEqual(info.returncode, 1)

    def assert_pw(self, serial, password):
        info = self.runPwgen(os.path.join(self.__datadir, self.TEST_SALT_FILE),
                             serial)
        out = info.stdout.decode('utf-8').strip()
        self.assertEqual(password, out)
        self.assertEqual(info.returncode, 0)

    def test_result_and_serial_disambiguation(self):
        """Test for expected output and serial number disambiguation."""
        # all these serial numbers should be equivalent based on our disambiguation
        pw = '59da62dfc8ee2fa2'

        # all letters
        self.assert_pw('ABC-DEF-GHI-JKL-MNO-PQR-STU-VWX-YZ', pw)

        # swap letters for similar-looking numbers
        self.assert_pw('ABC-DEF-GHI-JKL-MN0-PQR-STU-VWX-YZ', pw)
        self.assert_pw('ABC-DEF-6H1-JK1-MN0-PQR-STU-VWX-Y2', pw)

        # various hypen, underscore, and space manipulation
        self.assert_pw('ABC-DEFGHI-JKL-MNOPQR-STU-VWX-YZ', pw)
        self.assert_pw('AB-CD-EF-GHI-JKL-MNO-PQR-ST-UVW-XYZ', pw)
        self.assert_pw('ABCDEFGHIJKLMNOPQRSTUVWXYZ', pw)
        self.assert_pw('ABC_DEF_GHI_JKL_MNO_PQR_STU_VWX_YZ', pw)
        self.assert_pw('ABC DEF GHI JKL MNO PQR STU VWX YZ', pw)

        # lower case
        self.assert_pw('abc-def-ghi-jkl-mno-pqr-stu-vwx-yz', pw)

        # combined variation
        self.assert_pw('abCDef_gh1jk-lmn0pq-rstuvwxyz', pw)

    def test_assert_diff_passwords(self):
        """Test that substantially-different inputs yield different outputs."""

        self.assert_pw('ABC-DEF-GHI-JKL-MNO-PQR-STU-VWX-YZ', '59da62dfc8ee2fa2')
        self.assert_pw('1234567890', 'd94ec8b83e8b0546')
        self.assert_pw('12345ABCDE', 'c1eed53ab00069d0')

    def assert_invalid_chars(self, serial):
        info = self.runPwgen(os.path.join(self.__datadir, self.TEST_SALT_FILE), serial)
        out = info.stdout.decode('utf-8').strip()
        self.assertIn('serial number has unexpected characters', out)
        self.assertEqual(info.returncode, 1)

    def test_assert_expected_chars(self):
        """Test that input only contains a strict set of characters."""

        self.assert_invalid_chars('#123-ABC')
        self.assert_invalid_chars('(invalid)')
        self.assert_invalid_chars('ABC-DEF-123-.')
        self.assert_invalid_chars('ABC-DEF-123\n')
        self.assert_invalid_chars('\tABC-DEF-123')

if __name__ == '__main__':
    unittest.main(testRunner=taptestrunner.TAPTestRunner())
